@@ -236,12 +236,12 @@ If ARG is 1, prompt for a group name to find the posting style."
   (let ((gnus-newsgroup-name
 	 (if arg
 	     (if (= 1 (prefix-numeric-value arg))
-		 (completing-read "Use style of group: " gnus-active-hashtb nil
+		 (completing-read "Use posting style of group: "
+				  gnus-active-hashtb nil
 				  (gnus-read-active-file-p))
 	       (gnus-group-group-name))
 	   "")))
-    (gnus-setup-message 'message (message-mail))
-    ))
+    (gnus-setup-message 'message (message-mail))))
 
 (defun gnus-group-post-news (&optional arg)
   "Start composing a news message.
@@ -395,7 +395,7 @@ header line with the old Message-ID."
 	  ;; Delete the headers from the displayed articles.
 	  (set-buffer gnus-article-copy)
 	  (delete-region (goto-char (point-min))
-			 (or (search-forward "\n\n" nil t) (point)))
+			 (or (search-forward "\n\n" nil t) (point-max)))
 	  ;; Insert the original article headers.
 	  (insert-buffer-substring gnus-original-article-buffer beg end)
 	  (article-decode-encoded-words)))
@@ -527,7 +527,7 @@ If SILENT, don't prompt the user."
      ((and (eq gnus-post-method 'current)
 	   (not (eq (car group-method) 'nndraft))
 	   (not arg))
-      group-method) 
+      group-method)
      ((and gnus-post-method
 	   (not (eq gnus-post-method 'current)))
       gnus-post-method)
@@ -615,10 +615,17 @@ If FULL-HEADERS (the prefix), include full headers when forwarding."
   (interactive "P")
   (gnus-setup-message 'forward
     (gnus-summary-select-article)
-    (set-buffer gnus-original-article-buffer)
-    (let ((message-included-forward-headers
-	   (if full-headers "" message-included-forward-headers)))
-      (message-forward post))))
+    (let (text)
+      (save-excursion
+	(set-buffer gnus-original-article-buffer)
+	(setq text (buffer-string)))
+      (set-buffer (gnus-get-buffer-create " *Gnus forward*"))
+      (erase-buffer)
+      (insert text)
+      (run-hooks 'gnus-article-decode-hook)
+      (let ((message-included-forward-headers
+	     (if full-headers "" message-included-forward-headers)))
+	(message-forward post)))))
 
 (defun gnus-summary-resend-message (address n)
   "Resend the current article to ADDRESS."
@@ -668,7 +675,8 @@ The current group name will be inserted at \"%s\".")
 	(gnus-summary-select-article)
 	(set-buffer gnus-original-article-buffer)
 	(if (and (<= (length (message-tokenize-header
-			      (setq newsgroups (mail-fetch-field "newsgroups"))
+			      (setq newsgroups
+				    (mail-fetch-field "newsgroups"))
 			      ", "))
 		     1)
 		 (or (not (setq followup-to (mail-fetch-field "followup-to")))
@@ -973,7 +981,7 @@ this is a reply."
 	  (and gnus-newsgroup-name
 	       (gnus-group-find-parameter
 		gnus-newsgroup-name 'gcc-self)))
-	 result 
+	 result
 	 (groups
 	  (cond
 	   ((null gnus-message-archive-method)
@@ -1077,7 +1085,7 @@ this is a reply."
 	    (if (and (not (stringp (car attribute)))
 		     (not (eq 'body (car attribute)))
 		     (not (setq variable
-				(cdr (assq (car attribute) 
+				(cdr (assq (car attribute)
 					   gnus-posting-style-alist)))))
 		(message "Couldn't find attribute %s" (car attribute))
 	      ;; We get the value.
